class SQLNode(object):
	
	def __and2__(self, other):
		return SQLBinop("and", self, other)

	def __rand2__(self, other):
		return SQLBinop("and", other, self)

	def __eq__(self, other):
		return SQLBinop("=", self, other)


class Table(SQLNode):

	def __init__(self, name):
		self.__tablename__ = name
		
	def __getattr__(self, name):
		return SQLAttr(self, name)
	
	def __sql__(self):
		return self.__tablename__


class SQLBinop(SQLNode):

	def __init__(self, op, opnd1, opnd2):
		self.op = op
		self.opnd1 = opnd1
		self.opnd2 = opnd2

	def __sql__(self):
		return "(%s %s %s)" % (sql(self.opnd1), self.op, sql(self.opnd2))


class SQLAttr(SQLNode):

	def __init__(self, table, name):
		self.table = table
		self.name = name
	
	def __sql__(self):
		return "%s.%s" % (sql(self.table), self.name)


class SQLSelect(SQLNode):

	def __init__(self, targets):
		self.targets = targets
		self.where_clause = None

	def where(self, expr):
		self.where_clause = expr
		return self
	
	def __sql__(self):
		result = "SELECT %s" % ", ".join([sql(target) for target in self.targets])
		if self.where_clause:
			result = "%s WHERE %s" % (result, sql(self.where_clause))
		return result


def sql(expr):
	if isinstance(expr, SQLNode):
		return expr.__sql__()
	elif isinstance(expr, str):
		return "'%s'" % expr.replace("'", "''")
	else:
		return str(expr)


def select(*targets):
	return SQLSelect(targets)

#--------------------------------------------------------------------------------

dishes = Table("dishes")
customers = Table("customers")
orders = Table("orders")

query = select(customers.name, dishes.price, orders.amount).where(
	customers.cust_id == orders.cust_id and orders.dish_id == dishes.dish_id
	and dishes.name == "Spam, Eggs, Sausages and Spam")

print repr(query)
print sql(query)
